#include "stdafx.h"
#include "vtWave.h"


/**************************************************************************
 *  Default constructor                                                   *
 **************************************************************************/
VTRecordWave::VTRecordWave() : VTWave()
{
	m_pWhdrIn = NULL;
}


/**************************************************************************
 *  Constructor                                                           *
 **************************************************************************/
VTRecordWave::VTRecordWave(HWND hwnd, int iFormat)
   :VTWave(hwnd, iFormat)
{
	m_pWhdrIn = NULL;
}


/**************************************************************************
 *  Destructor                                                   *
 **************************************************************************/
VTRecordWave::~VTRecordWave()
{
	// If wave input device still open
	// close it
	if(m_hwIn)
	   waveInClose(m_hwIn);

	// Release the memory for wave header
	if(m_pWhdrIn)
       CleanMemory();
}


/**************************************************************************
 *  Allocate memory for the header structure of wave input                *
 **************************************************************************/
BOOL VTRecordWave::AllocMemory(void)
{
	//Allocated buffer for WAVEHDR structure
	m_pWhdrIn = (WAVEHDR* )HeapAlloc( GetProcessHeap(),                         
	                      HEAP_ZERO_MEMORY,                         
	                      sizeof(WAVEHDR) );               
	// Success
	if(m_pWhdrIn)                                                  
	{         
		// Allocate buffer of wave data in wave header structure
	    m_pWhdrIn->lpData = (char*)HeapAlloc( GetProcessHeap(),             
	                                  HEAP_ZERO_MEMORY,             
	                                  VT_MAX);                  
		
		// Set wave data buffer length
	    m_pWhdrIn->dwBufferLength = VT_MAX;

		// Set the flag of wave header structure
		m_pWhdrIn->dwFlags = 0;
	}
	else // An error ocurred
	{
        MessageBox(m_hWnd, "Failed to allocate memory for input device", 
			       "Error in allocating input device memory", MB_OK);
	    return FALSE;
	}

	// OK
	return TRUE;
}


/**************************************************************************
 *  Clearn the memory of the header structure of wave input               *
 **************************************************************************/
void VTRecordWave::CleanMemory(void)
{
	// If not clean the buffer
	if(m_pWhdrIn)
	{
		// First clean the buffer of wave data
		if(m_pWhdrIn->lpData)
            HeapFree(GetProcessHeap(), 0, m_pWhdrIn->lpData);                 

		// Clean all
        HeapFree(GetProcessHeap(), 0, m_pWhdrIn);
        m_pWhdrIn = NULL;
	}
}


/**************************************************************************
 *  Initialize and open the wave input device                             *
 **************************************************************************/
BOOL VTRecordWave::Initialize(void)
{
    WAVEINCAPS    wic;
    WAVEFORMATEX  wfx;
    UINT          uiDevID;
    MMRESULT      rc;
    UINT          nMaxDev = ::waveInGetNumDevs();
    char          sError[129];

	//Check audio device and open it
    for(uiDevID = 0; uiDevID < nMaxDev; uiDevID++)
    {
       // Get input device caps
	   rc = ::waveInGetDevCaps(uiDevID, &wic, sizeof(wic));
	   if(rc == MMSYSERR_NOERROR)
       {
          // Set the correct format for wave input device 
	      wfx.nChannels = VTWAVECHANNEL[m_iFormat];  //1: mono, 2, stereo
		  wfx.nSamplesPerSec  = VTWAVEFREQ[m_iFormat];
          wfx.wFormatTag      = WAVE_FORMAT_PCM;
		  wfx.wBitsPerSample  = VTWAVEBITS[m_iFormat];
          wfx.nBlockAlign     = wfx.nChannels*wfx.wBitsPerSample/8;
          wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;  
       	  wfx.cbSize          = 0;
		  
		  // Open the wave input device
          rc = waveInOpen(&m_hwIn, uiDevID, &wfx, (DWORD)m_hWnd, 
			              0, CALLBACK_WINDOW);                                          

	      if (rc == MMSYSERR_NOERROR)                              
	           break;                                               
	      else // Failed                                                    
	      {             
			   // Get the error information
	           waveInGetErrorText(rc, sError, 128),                    
	           MessageBox(m_hWnd, sError, "Error in input device opening", MB_OK);
	           return FALSE;
	      }                                                        
       }
    }

	// Device not found, error condition
    if(m_hwIn == NULL)
    {
       MessageBox(m_hWnd, "Can't find correct input device to open", 
		          "Error in e open recording device", MB_OK);
	   return FALSE;
    }

	// OK
    return TRUE;
}


/**************************************************************************
 *  Start to record the sound                                             *
 **************************************************************************/
BOOL VTRecordWave::StartRecord(void)
{
    MMRESULT rc;
    char     sError[129];

	// Prepare and open wave input device
	Initialize();

	// Prepare buffer blocks and add to input queue
    rc = waveInPrepareHeader(m_hwIn, m_pWhdrIn, sizeof(WAVEHDR));
	                                                                
	// Add buffers to the input queue                            
    if(rc == MMSYSERR_NOERROR)
	   rc = waveInAddBuffer(m_hwIn, m_pWhdrIn, sizeof(WAVEHDR));
	else  // Failed                                                              
    {              
	   // Get error information	
       waveInGetErrorText(rc, sError, 128),                        
	   MessageBox(m_hWnd, sError, "Error in Adding buffer to input device", MB_OK);
	   // Stop!
       StopRecord();  
	   return FALSE;                                              
	}                                                               
	                                                                
	// Start recording                                              
	rc = waveInStart(m_hwIn);
    
	// Failed
	if(rc != MMSYSERR_NOERROR)
	{
       // Get error information
       waveInGetErrorText(rc, sError, 128),                        
	   MessageBox(m_hWnd, sError, "Error in starting input device", MB_OK);
	   // Stop!
       StopRecord();  
	   return FALSE;                                              
	}

	// OK
	return TRUE;
}


/**************************************************************************
 *  Stop recording the sound                                              *
 **************************************************************************/
void VTRecordWave::StopRecord(void)
{
	if(m_hwIn)
	{
		// Stop the input device
	    waveInStop(m_hwIn);

	    // Unprepare headers                                                      
        waveInUnprepareHeader(m_hwIn, m_pWhdrIn, sizeof(WAVEHDR));

	    // Close the wave input device handle
	    waveInClose(m_hwIn);
	    m_hwIn = NULL;
	}
}


/**************************************************************************
 *  Reset input device to record new input sound data                     *
 **************************************************************************/
BOOL VTRecordWave::ResetRecord(void)
{
    MMRESULT rc;
    char     sError[129];

	//Stop first
	waveInStop(m_hwIn);

	// Unprepare headers                                                      
    waveInUnprepareHeader(m_hwIn, m_pWhdrIn, sizeof(WAVEHDR));
	
	// Prepare buffer blocks and add to input queue
    rc = waveInPrepareHeader(m_hwIn, m_pWhdrIn, sizeof(WAVEHDR));       
 	    
	// Add buffer to the input queue
    if(rc == MMSYSERR_NOERROR)                                  
       rc = waveInAddBuffer(m_hwIn, m_pWhdrIn, sizeof(WAVEHDR));       
 	else // Failed
	{
       // Get the error information  
       waveInGetErrorText(rc, sError, 128),                        
	   MessageBox(m_hWnd, sError, "Error in preparing buffer when resetting", MB_OK);
	   // Stop!
       StopRecord();  
	   return FALSE;                                              
	}	
   	

	if(rc == MMSYSERR_NOERROR)
	{
	   // Start recording
	   rc = waveInStart(m_hwIn);
	}
    else // Failed to adding buffer
	{  
	   // Get error information 	
       waveInGetErrorText(rc, sError, 128),                        
	   MessageBox(m_hWnd, sError, "Error in adding buffer when resetting", MB_OK);
	   // Stop!
       StopRecord();  
	   return FALSE;                                              
	}

	// Failed to start recording
	if(rc != MMSYSERR_NOERROR)
	{
       // Get error information
       waveInGetErrorText(rc, sError, 128),                        
	   MessageBox(m_hWnd, sError, "Error in starting input device when resetting", MB_OK);
	   // Stop!
       StopRecord();  
	   return FALSE;                                              
	}

	// OK
	return TRUE;
}

